﻿Shader "Custom/IceShader" {
	Properties {
      _MainTex ("Texture", 2D) = "white" {}
      _BumpMap ("Bumpmap", 2D) = "bump" {}
      _BaseColor ("Base Color", Color) = (1.0,1.0,1.0,0.0)
      _RimColor ("Rim Color", Color) = (0.26,0.19,0.16,0.0)
      _RimPower ("Rim Power", Range(0.5,8.0)) = 3.0
      _Specular ("Specular", Range(0.0,1.0)) = 1.0
      _SpecColor ("Specular Color", Color) = (0.8,0.8,0.8,1.0)
    }
    SubShader {
		Tags { "RenderType" = "Opaque" }
		LOD 400
		CGPROGRAM
		#pragma surface surf BlinnPhong
		#pragma target 3.0
		struct Input {
			float2 uv_MainTex;
			float2 uv_BumpMap;
			float3 viewDir;
		};
		sampler2D _MainTex;
		sampler2D _BumpMap;
		float4 _BaseColor;
		float4 _RimColor;
		float _RimPower;
		float _Specular;		
      
		// Albedo, Normal, Emission, Specular, Gloss, Alpha;

		void surf (Input IN, inout SurfaceOutput o) {
		  float4 ColorGloss = tex2D (_MainTex, IN.uv_MainTex);
		  
		  o.Albedo = lerp(ColorGloss.rgb, _BaseColor.rgb, _BaseColor.a);
		  o.Gloss = ColorGloss.a;
		  o.Specular = _Specular;
		  o.Normal = UnpackNormal (tex2D (_BumpMap, IN.uv_BumpMap));
		  
		  half rim = 1.0 - saturate(dot (normalize(IN.viewDir), o.Normal));
		  o.Emission = _RimColor.rgb * pow (rim, _RimPower);
		}
		ENDCG
    } 
    Fallback "Specular"
  }
