﻿Shader "Custom/OceanShader" {
	Properties {
      _MainTex ("Texture", 2D) = "white" {}
      _BumpMap ("Bumpmap", 2D) = "bump" {}
      _BaseColor ("Base Color", Color) = (1.0,1.0,1.0,0.0)
      _RimColor ("Rim Color", Color) = (0.26,0.19,0.16,0.0)
      _RimPower ("Rim Power", Range(0.5,8.0)) = 3.0
      _Specular ("Specular", Range(0.0,1.0)) = 1.0
      _SpecColor ("Specular Color", Color) = (0.8,0.8,0.8,1.0)
      _FogColor ("Fog Color", Color) = (0.5,0.5,0.9,1.0)
    }
    SubShader {
		Tags { "RenderType" = "Opaque" }
		CGPROGRAM
		#pragma surface surf Lambert vertex:vert finalcolor:FogColor
		#pragma target 3.0
		
		struct Input {
			float2 uv_MainTex;
			float2 uv_BumpMap;
			float3 viewDir;
			float3 customColor;
			float3 worldPos;
		};
		sampler2D _MainTex;
		sampler2D _BumpMap;
		float4 _BaseColor;
		float4 _RimColor;
		float _RimPower;
		float _Specular;
		float4 _FogColor;		
		
		void vert (inout appdata_full v, out Input o) {
		
			UNITY_INITIALIZE_OUTPUT(Input,o);
		
			float2 UVNorm = v.vertex.xy * 0.001f;
			float2 UVNorm2 = UVNorm.xy * 0.2731;
			
			float TimeScale = 0.7f;
			
			UVNorm += _Time * TimeScale * float2(0.7f, 0.7f) * 0.01f;
			UVNorm2 += _Time * TimeScale * float2(-1.0f, 0.0f) * 0.0062f;
			
			#if !defined(SHADER_API_OPENGL)
			
				float3 NormBis = UnpackNormal (tex2Dlod (_BumpMap, float4(UVNorm2,0,0)));
			
				float3 Norm = UnpackNormal (tex2Dlod (_BumpMap, float4(UVNorm + NormBis.xy * 0.05f,0,0)));
				
				//Norm.xy = Norm.xy + NormBis.xy;
				//Norm.z = Norm.z * NormBis.z;
				//Norm = normalize(Norm);
			#else
				float3 Norm = float3(0.0f, 0.0f, 1.0f);
			#endif
			
			float3 NewNorm = normalize(lerp(v.normal, Norm, 5.0f));
			v.normal.xyz = NewNorm.xyz;
			float3 Bitangent = cross(v.tangent, v.normal);
			v.tangent.xyz = normalize(cross(v.normal, Bitangent));
			
			
		
			v.vertex.z += (1.0f-Norm.z) * 20;
			
			o.customColor = NewNorm;
		}

		// Albedo, Normal, Emission, Specular, Gloss, Alpha;

		void surf (Input IN, inout SurfaceOutput o) {
			float4 ColorGloss = tex2D (_MainTex, IN.uv_MainTex);

			o.Albedo = lerp(ColorGloss.rgb, _BaseColor.rgb, _BaseColor.a);
			o.Gloss = ColorGloss.a;
			o.Specular = _Specular;

			float TimeScale = 0.7f;
			
			float2 UVNorm = IN.uv_BumpMap;
			float2 UVNorm3 = UVNorm.xy * 0.43f;
			float2 UVNorm2 = UVNorm.yx * 0.175f;
			
			UVNorm += _Time * TimeScale * float2(0.7f, 0.7f) * 0.1f;
			UVNorm2 += _Time * TimeScale * float2(-1.0f, 0.0f) * 0.062f * 3.0f;
			UVNorm3 += _Time * TimeScale * float2(0.0f, 1.0f) * 0.087f;

			float3 NormBis = UnpackNormal (tex2D (_BumpMap, UVNorm2));
			float3 Norm1 = UnpackNormal (tex2D (_BumpMap, UVNorm + NormBis.xy * 0.05f));
			float3 Norm3 = UnpackNormal (tex2D (_BumpMap, UVNorm3 + NormBis.yx * 0.05f));
			
			o.Normal.xy = Norm1.xy+Norm3.xy;
			o.Normal.z = Norm1.z * Norm3.z;
			o.Normal = normalize(o.Normal);

			half rim = 1.0 - saturate(dot (normalize(IN.viewDir), o.Normal));
			o.Emission = _RimColor.rgb * pow (rim, _RimPower);

			//o.Emission = saturate(IN.customColor);
		}
		
		void FogColor (Input IN, SurfaceOutput o, inout fixed4 color)
		{
			float2 ScaledUV = IN.worldPos.xz * 0.0005f;
			fixed3 FogValue = 1.0f-saturate(sqrt(dot(ScaledUV, ScaledUV)));
			#ifdef UNITY_PASS_FORWARDADD
			FogValue = 0;
			#endif
			color.rgb = lerp (_FogColor.rgb, color.rgb, FogValue);
		}
		
		ENDCG
    } 
    Fallback "Specular"
  }

