﻿using UnityEngine;
using System.Collections;
using System;

public class VehiculeMesh : MonoBehaviour {

	public Transform target;
	public Transform mesh;

	public FollowingCamera CamActor;

	public float MinZHeight;
	public GUIText TimerText;
	public GUIText TimerTextShadow;

	float Timer;
	bool FinishedTrack;

	Quaternion OldRotation;
	Vector3 LastNormal;

	// Use this for initialization
	void Start () {
		LastNormal = Vector3.up;
		Timer = 0.0f;
		FinishedTrack = false;
	}
	
	// Update is called once per frame
	void FixedUpdate () {

		float DeltaTime = Time.fixedDeltaTime;

		if (MainMng.Instance != null) {
			if(MainMng.Instance.FinishCheckpoint >= MainMng.Instance.CheckpointNumber) {
				FinishedTrack = true;
			}
		}

		if (!FinishedTrack) {
			Timer += DeltaTime;
		}
		TimeSpan timeSpan = TimeSpan.FromSeconds(Timer);



		string timeText = string.Format("{0:D2}:{1:D2}:{2:D2}", timeSpan.Minutes, timeSpan.Seconds, timeSpan.Milliseconds/10);
		TimerText.text = timeText;
		TimerTextShadow.text = timeText;

		bool IsInSpace = (MainMng.Instance != null) ? MainMng.Instance.IsInSpace : false;
		bool WasInSpace = IsInSpace;

		if (MainMng.Instance != null) {
			if (MainMng.Instance.NextPortal != null) {
				IsInSpace = MainMng.Instance.NextPortal.PortalToSpace;
				MainMng.Instance.IsInSpace = IsInSpace;
			}
		}

		float ScaleMotion = 1600;
		
		bool Up = Input.GetKey(KeyCode.UpArrow);
		bool Down = Input.GetKey(KeyCode.DownArrow);
		bool Left = Input.GetKey(KeyCode.LeftArrow);
		bool Right = Input.GetKey(KeyCode.RightArrow);
		
		float MagnAdded = 0.0f;
		Vector3 MotionAdded = new Vector3 (0.0f, 0.0f, 0.0f);
		
		Vector3 ForwardVec = Vector3.Normalize (transform.forward);
		Vector3 RightVec = Vector3.Normalize (transform.right);
		float Magnitude = target.transform.rigidbody.velocity.magnitude;
		float DirectionnalSpeed = Vector3.Dot(ForwardVec, target.transform.rigidbody.velocity);
		
		if (Up) {
			MagnAdded += ScaleMotion * DeltaTime;
		}
		if (Down) {
			//MagnAdded += -DirectionnalSpeed * ScaleMotion * DeltaTime;
		}
		
		MotionAdded += ForwardVec * MagnAdded;

		float ScaleRotation = 0.7f;
		float RotLimit = 30;
		float ScaleMotionRot = 500;
		
		float Speed = (Mathf.Min(Magnitude, RotLimit) + 0.1f*ScaleMotion*DeltaTime);
		float Rotation = 0.0f;
		if (Left) {
			Rotation += -ScaleRotation * Speed;
			MotionAdded -= RightVec * Mathf.Clamp01(DirectionnalSpeed/10.0f) * ScaleMotionRot * DeltaTime;
		}
		if (Right) {
			Rotation += ScaleRotation * Speed;
			MotionAdded += RightVec * Mathf.Clamp01(DirectionnalSpeed/10.0f) * ScaleMotionRot * DeltaTime;
		}

		float FloorFactor = 0.0f;
		float FloorTestDist = 5.0f;
		float MotionMaxDist = 1.5f;

		bool IsTrackMagnet = false;

		Vector3 UpDir = IsInSpace ? Vector3.down : Vector3.up;
		Vector3 MagnetNormal = UpDir;

		RaycastHit hit;
		Vector3 fwd = -transform.up;
		if (Physics.Raycast (transform.position, fwd, out hit, FloorTestDist)) {
		
			MagnetNormal = Vector3.Normalize (hit.normal);
			IsTrackMagnet = hit.collider.CompareTag("Track_Magnet");

			if(hit.distance<MotionMaxDist || IsTrackMagnet)
			{
				if(!hit.collider.isTrigger)
				{
					FloorFactor = 1.0f - Mathf.Clamp01 (hit.distance / MotionMaxDist);
					LastNormal = Vector3.Normalize (hit.normal);
				}
				//LastNormal = Vector3.Slerp (LastNormal, Vector3.Normalize (hit.normal), DeltaTime * 2.0f);
			}

		} else {

			LastNormal = Vector3.Slerp (LastNormal, UpDir, DeltaTime * 0.5f);
			
		}

		Vector3 MagnetNewPosition = Vector3.zero;
		//if (IsTrackMagnet) {
		{
			IsTrackMagnet = false;

			RaycastHit hitMagnet;
			if (Physics.Raycast (target.transform.position, -MagnetNormal, out hitMagnet, 10.0f)) {
							
				if(hit.collider && hit.collider.CompareTag("Track_Magnet")) {
					IsTrackMagnet = true;
					MagnetNewPosition = - MagnetNormal * Mathf.Max(0.0f, hitMagnet.distance - 0.5f);
				}
				
			}
		}
	
		if (IsTrackMagnet) {

			target.rigidbody.useGravity = false;
			//target.transform.rigidbody.MovePosition(MagnetNewPosition);
			target.transform.rigidbody.AddForce ((MagnetNewPosition) * DeltaTime * 1000.0f);

		} else {

			if(IsInSpace) {
				target.rigidbody.useGravity = false;

				target.transform.rigidbody.AddForce(Vector3.up * 9.8f);

			} else {
				target.rigidbody.useGravity = true;
			}
		}

		transform.Rotate (new Vector3 (0.0f, Rotation, 0.0f));
		target.transform.rigidbody.AddForce(MotionAdded * FloorFactor);
		
		if (Down) {
			float Brake = Mathf.Min(2.0f*DeltaTime, 1.0f);
			target.transform.rigidbody.AddForce(-FloorFactor*Brake*target.transform.rigidbody.velocity/DeltaTime);
		}
		
		transform.position = target.transform.position;
		mesh.transform.position = transform.position;

		Quaternion NewRotation = transform.rotation;

        Quaternion AlignRotation2 = Quaternion.FromToRotation(Vector3.Normalize(transform.up), LastNormal);
        NewRotation = AlignRotation2*NewRotation;

		OldRotation = Quaternion.Slerp (OldRotation, NewRotation, DeltaTime * 10.0f);

		mesh.rotation = OldRotation;
		transform.rotation = OldRotation;

		mesh.Rotate (-90,0,0);

		if(MainMng.Instance != null) {
			if(MainMng.Instance.NextPortal != null) {
				Portal CurrentCheckpoint = MainMng.Instance.NextPortal;
				target.transform.position = CurrentCheckpoint.Goal.position;
				Vector3 EulerRot = CurrentCheckpoint.Goal.rotation.eulerAngles;
				transform.rotation = Quaternion.Euler(EulerRot.x, EulerRot.y, 0.0f);
				OldRotation = transform.rotation;
				LastNormal = Vector3.up;
				target.rigidbody.velocity = Vector3.zero;
				target.rigidbody.angularVelocity = Vector3.zero;

				if(IsInSpace) {
					target.transform.rigidbody.AddForce(Vector3.down * 1000.0f);
				} else {
					target.transform.rigidbody.AddForce(Vector3.up * 600.0f + Vector3.forward * 300.0f);
				}

				CamActor.NoInterpolate = true;
				MainMng.Instance.NextPortal = null;
			}

			if (!IsInSpace && !WasInSpace && transform.position.y < MinZHeight) {
				MainMng.Instance.NeedKill = true;
			}
			
			if (MainMng.Instance.NeedKill) {
				MainMng.Instance.NeedKill = false;

				if(MainMng.Instance.CurrentCheckpoint != null) {
					Checkpoint CurrentCheckpoint = MainMng.Instance.CurrentCheckpoint;
					target.transform.position = CurrentCheckpoint.transform.position;
					//target.transform.rotation = Quaternion.FromToRotation(Vector3.up, CurrentCheckpoint.transform.up);
					//target.transform.rotation = CurrentCheckpoint.transform.rotation;
					Vector3 EulerRot = CurrentCheckpoint.transform.rotation.eulerAngles;
					transform.rotation = Quaternion.Euler(EulerRot.x, EulerRot.y, 0.0f);
					//transform.rotation = CurrentCheckpoint.transform.rotation;
					OldRotation = transform.rotation;
					LastNormal = Vector3.up;
					target.rigidbody.velocity = Vector3.zero;
					target.rigidbody.angularVelocity = Vector3.zero;

					if(CurrentCheckpoint.FinalCheckpoint) {
						FinishedTrack = true;
					}

					CamActor.NoInterpolate = true;
					MainMng.Instance.IsInSpace = CurrentCheckpoint.IsInSpace;

				} else if(MainMng.Instance.Start != null) {
					StartingPoint CurrentCheckpoint = MainMng.Instance.Start;
					target.transform.position = CurrentCheckpoint.transform.position;
					Vector3 EulerRot = CurrentCheckpoint.transform.rotation.eulerAngles;
					transform.rotation = Quaternion.Euler(EulerRot.x, EulerRot.y, 0.0f);
					OldRotation = transform.rotation;
					LastNormal = Vector3.up;
					target.rigidbody.velocity = Vector3.zero;
					target.rigidbody.angularVelocity = Vector3.zero;

					CamActor.NoInterpolate = true;
					MainMng.Instance.IsInSpace = false;
				}
			}
		}
	}
}
